# Copyright 2024 RnD Center "ELVEES", JSC

import glob
import os
import matplotlib.pyplot as plt
import pandas as pd
import subprocess


def test_signal_lib_fft(request):
    tests_names = [
        "test_FLOAT_FFT_1D_EXT_EXT",
        "test_FLOAT_FFT_1D_EXT_INT",
        "test_FLOAT_FFT_2D_EXT_EXT",
        "test_FLOAT_FFT_2D_EXT_INT",
        "test_FLOAT_IFFT_1D_EXT_EXT",
        "test_FLOAT_IFFT_1D_EXT_INT",
        "test_FLOAT_IFFT_2D_EXT_EXT",
        "test_FLOAT_IFFT_2D_EXT_INT",
        "test_HFLOAT_FFT_1D_EXT_EXT",
        "test_HFLOAT_FFT_1D_EXT_INT",
        "test_HFLOAT_FFT_2D_EXT_EXT",
        "test_HFLOAT_FFT_2D_EXT_INT",
        "test_HFLOAT_IFFT_1D_EXT_EXT",
        "test_HFLOAT_IFFT_1D_EXT_INT",
        "test_HFLOAT_IFFT_2D_EXT_EXT",
        "test_HFLOAT_IFFT_2D_EXT_INT",
        "test_FRACTIONAL_FFT_1D_EXT_EXT",
        "test_FRACTIONAL_FFT_1D_EXT_INT",
        "test_FRACTIONAL_FFT_2D_EXT_EXT",
        "test_FRACTIONAL_FFT_2D_EXT_INT",
        "test_FRACTIONAL_IFFT_1D_EXT_EXT",
        "test_FRACTIONAL_IFFT_1D_EXT_INT",
        "test_FRACTIONAL_IFFT_2D_EXT_EXT",
        "test_FRACTIONAL_IFFT_2D_EXT_INT",
    ]

    for name in tests_names:
        binary_path = os.path.join(os.path.dirname(__file__) + "/elf/", name + ".elf")
        subprocess.run(
            ["elcorecl-run", "-e", binary_path],
            check=True,
            timeout=100,
        )

    fig, axs = plt.subplots(nrows=3, ncols=4, figsize=(42, 13))
    fig.suptitle("FFT elcore50 performance")

    dsp_freq = (
        int(
            subprocess.run(
                ["cat", "/sys/kernel/debug/clk/dsp0_clk/clk_rate"],
                capture_output=True,
                text=True,
                check=True,
            ).stdout
        )
        * 1e-6
    )

    for i, test_name in enumerate(tests_names):
        plt_float = pd.read_csv(f"{test_name}.csv", index_col=0)
        time_plt_float = plt_float["TIC_CNTR"] / dsp_freq
        axs[i // 8, (i // 2) % 4].plot(time_plt_float, label="EXT_INT" if (i % 2) else "EXT_EXT")
        if i % 2:
            axs[i // 8, (i // 2) % 4].grid()
        axs[i // 8, (i // 2) % 4].set_yscale("log")

    axs[0, 0].set_ylabel("FLOAT")
    axs[1, 0].set_ylabel("HFLOAT")
    axs[2, 0].set_ylabel("FRACTIONAL")

    axs[0, 0].set_title("FFT 1D")
    axs[0, 1].set_title("FFT 2D")
    axs[0, 2].set_title("IFFT 1D")
    axs[0, 3].set_title("IFFT 2D")

    axs[0, 3].set_ylabel("time, \u03BCs")
    axs[0, 3].set_xlabel("(I)FFT size, complex elements")
    axs[0, 3].legend()

    artifacts_path = request.config.getoption("--artifacts-path")
    fig.savefig(f"{artifacts_path}/elcore50_fft_performance.png")
    plt.close()

    # Move .csv files to artifacts
    files_to_move = glob.glob("test_*FFT*.csv")
    for file in files_to_move:
        subprocess.run(["mv", file, f"{artifacts_path}/{file}"], check=True, timeout=10)


def test_signal_lib_conv_1d(request):
    data_locs = ["DDR", "XYRAM"]
    binary_path = os.path.join(os.path.dirname(__file__) + "/elf/", "test_conv_1d.elf")
    for mem in data_locs:
        subprocess.run(
            ["elcorecl-run", "-e", binary_path, "--", "-m", f"{mem}"],
            check=True,
            timeout=300,
        )
    # Move .csv files to artifacts
    artifacts_path = request.config.getoption("--artifacts-path")
    files_to_move = glob.glob("test_conv_1d_*.csv")
    for file in files_to_move:
        subprocess.run(["mv", file, f"{artifacts_path}/{file}"], check=True, timeout=10)


def test_signal_lib_conv_2d():
    data_locs = ["DDR", "XYRAM"]
    matrices_sizes = [
        ["16", "16", "16", "16"],
        ["16", "32", "32", "64"],
        ["256", "128", "32", "16"],
        ["256", "128", "128", "256"],
    ]
    gen_script_path = os.path.join(
        os.path.dirname(__file__) + "/test_conv/", "gen_conv_test_data.py"
    )
    test_data_path = os.path.dirname(__file__) + "/test_conv/test_data"
    os.makedirs(test_data_path, exist_ok=True)
    binary_path = os.path.join(os.path.dirname(__file__) + "/elf/", "test_conv_2d.elf")
    for size in matrices_sizes:
        subprocess.run(
            [
                ".venv/bin/python",
                gen_script_path,
                "-r0",
                size[0],
                "-c0",
                size[1],
                "-r1",
                size[2],
                "-c1",
                size[3],
                "-l",
                test_data_path,
            ],
            check=True,
            timeout=600,
        )
        for mem in data_locs:
            subprocess.run(
                [
                    "elcorecl-run",
                    "-e",
                    binary_path,
                    "--",
                    "-i",
                    test_data_path + "/input_data0.bin",
                    test_data_path + "/input_data1.bin",
                    "-o",
                    test_data_path + "/output_data0.bin",
                    "-m",
                    f"{mem}",
                    "-r0",
                    size[0],
                    "-c0",
                    size[1],
                    "-r1",
                    size[2],
                    "-c1",
                    size[3],
                ],
                check=True,
                timeout=300,
            )
    subprocess.run(
        ["rm", "-rf", test_data_path],
        check=True,
        timeout=100,
    )
